@@ -7,12 +7,13 @@ import os
 import random
 import sys
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Union, no_type_check
+from typing import Any, Dict, List, Optional, Tuple, Union, no_type_check
 from warnings import warn
 
 import colorama
 import pandas as pd
 import requests
+from tqdm import tqdm
 from typeguard import typechecked
 
 from gantry.config import Config
@@ -40,7 +41,6 @@ from gantry.logger.utils import (
     _is_not_empty,
     _log_exception,
     get_file_linecount,
-    progressbar,
 )
 from gantry.query.core.utils import check_response
 from gantry.serializers import serializable_value
@@ -63,7 +63,8 @@ class Gantry:
         Args:
             logs_store (BaseLogStore): The log store to use.
             environment (optional, str): Set the value for the environment label attached to data
-                instrumented by this client.
+                instrumented by this client. This value can be overridden for each record using
+                tags.
             logging_level (options: str): Configure logging level for Gantry system.
         """
         self.context = {"environment": environment}
@@ -271,8 +272,8 @@ class Gantry:
             block_read = functools.partial(f_in.read, CHUNK_SIZE)
             block_iterator = iter(block_read, "")
             part_num = 1  # AWS S3 needs part num to start with 1.
-            for block in progressbar(
-                block_iterator, prefix="Uploading to Gantry Bucket: ", count=total_num_parts
+            for block in tqdm(
+                block_iterator, desc="Uploading to Gantry Bucket: ", total=total_num_parts
             ):
                 # TODO: parallelize this client side?
                 response = requests.put(signed_urls[part_num - 1], data=block)
@@ -309,10 +310,10 @@ class Gantry:
         feedbacks: Optional[Union[List[dict], pd.DataFrame]] = None,
         ignore_inputs: Optional[List[str]] = None,
         timestamps: Optional[List[datetime.datetime]] = None,
-        reference_name: Optional[str] = None,
         sort_on_timestamp: bool = False,
         sample_rate: float = 1.0,
         as_batch: bool = False,
+        tags: Optional[Dict[str, str]] = None,
     ) -> None:
         """
         Function to record a batch of events containing predictions (inputs and outputs),
@@ -337,6 +338,7 @@ class Gantry:
                 version=1,
                 feedback_keys=["A"],
                 feedbacks=[{'B': 'bar'}, {'B': 'foo'}],
+                tags={"env": "environment1"}
             )
                 ...
 
@@ -358,8 +360,6 @@ class Gantry:
             feedback_ids (optional, List[dict]): A list of prediction feedback ids. The i-th entry
                 corresponds to the argument `feedback_ids` in
                 :meth:`gantry.client.Gantry.log_predictions` for the i-th prediction event.
-                **IMPORTANT: Only use this parameter if logging feedback only (no predictions).
-                If inputs/outputs were also passed in this method, this parameter will be ignored.**
             feedbacks (Union[List[dict], pd.Dataframe]): A list of feedbacks. `feedbacks[i]`
                 is a dict of the features for the i-th prediction to be logged.
             ignore_inputs (optional, List[str]): A list of names of input features that should not
@@ -368,14 +368,16 @@ class Gantry:
                 If specified, `timestamps[i]` should be the timestamp for the i-th prediction.
                 If timestamps = None (default), then the prediction timestamp defaults to the
                 time when `log_records` is called.
-            reference_name(optional, str): tag the data being pushed to later user in the
-                dashboard.
             sort_on_timestamp (bool, defaults to False): Works when timestamps are provided.
                 Sort using the given timestamp. Default to False.
             sample_rate: Used for downsampling. The probability as a float that each record
                 will be sent to Gantry.
             as_batch (bool, defaults to False): Whether to add batch metadata and tracking
                 in the 'batch' section of the dashboard
+            tags (optional, Optional[Dict[str, str]]): A tag is a label that you assign to your
+                data. E.g. you can specify which environment the data belongs to by setting "env"
+                tag like this tags = {"env": "environment1"} if not assigned we will use Gantry
+                client's environment value as the default environment.
 
         """
         some_preds_exist = _is_not_empty(inputs) or _is_not_empty(outputs)
@@ -398,10 +400,10 @@ class Gantry:
                 feedback_keys=feedback_keys,
                 ignore_inputs=ignore_inputs,
                 timestamps=timestamps,
-                reference_name=reference_name,
                 sort_on_timestamp=sort_on_timestamp,
                 sample_rate=sample_rate,
                 as_batch=as_batch,
+                tags=tags,
             )
         elif preds_exist:
             return self.log_predictions(
@@ -413,10 +415,10 @@ class Gantry:
                 ignore_inputs,
                 feedback_ids,
                 timestamps,
-                reference_name,
                 sort_on_timestamp,
                 sample_rate,
                 as_batch,
+                tags,
             )
         elif feedbacks_exist:
             return self.log_feedback(
@@ -428,6 +430,7 @@ class Gantry:
                 sort_on_timestamp,
                 sample_rate,
                 as_batch,
+                tags,
             )
         else:
             logger.error("Tried to log_records without prediction and without feedback")
@@ -440,6 +443,20 @@ class Gantry:
         """
         return self.log_store.ping()
 
+    def _get_env_and_context(self, tags: Optional[Dict]) -> Tuple[str, Dict]:
+        if tags and tags.get("env"):
+            return tags["env"], {"environment": tags.get("env")}
+        else:
+            return self.environment, self.context
+
+    @_log_exception
+    def ready(self) -> bool:
+        """
+        Checks if API is ready to receive traffic.
+        Returns True if ready, False otherwise.
+        """
+        return self.log_store.ready()
+
     @_log_exception
     def _log_predictions_and_feedbacks(
         self,
@@ -451,10 +468,10 @@ class Gantry:
         feedback_keys: Optional[List[str]] = None,
         ignore_inputs: Optional[List[str]] = None,
         timestamps: Optional[List[datetime.datetime]] = None,
-        reference_name: Optional[str] = None,
         sort_on_timestamp: bool = False,
         sample_rate: float = 1.0,
         as_batch: bool = False,
+        tags: Optional[Dict[str, str]] = None,
     ) -> None:
         """Internal method to log batch of predictions AND feedbacks"""
         if any(_is_empty(d) for d in [inputs, outputs, feedbacks]):
@@ -494,17 +511,14 @@ class Gantry:
         feedbacks = serializable_value(feedbacks)
         serialized_version: List[Union[int, str]] = serializable_value(version)
 
-        tags = {}
-        if reference_name:
-            tags["env"] = "reference"
-            tags["ref_name"] = reference_name
+        environment, context = self._get_env_and_context(tags)
 
         timestamp_idx = _create_timestamp_idx(sort_on_timestamp, timestamps, size)
 
         events = _build_prediction_and_feedback_events(
             application=application,
-            context=self.context,
-            environment=self.environment,
+            context=context,
+            environment=environment,
             timestamp_idx=timestamp_idx,
             tags=tags,
             version=serialized_version,
@@ -547,6 +561,7 @@ class Gantry:
         ignore_inputs: Optional[List[str]] = None,
         timestamp: Optional[datetime.datetime] = None,
         sample_rate: float = 1.0,
+        tags: Optional[Dict[str, str]] = None,
     ) -> None:
         """
         Function to record an event, regardless of inputs. Allows client to pass in inputs,
@@ -570,6 +585,7 @@ class Gantry:
                 version=1,
                 feedback_keys=["A"],
                 feedback_id={'A': 100},
+                tags = {"env":"environment1"}
             )
                 ...
 
@@ -599,6 +615,10 @@ class Gantry:
                 called.
             sample_rate (optional, float): Specify the probability as a float that the event will
                 be sent to Gantry.
+            tags (optional, Optional[Dict[str, str]]): A tag is a label that you assign to your
+                data. E.g. you can specify which environment the data belongs to by setting "env"
+                tag like this tags = {"env": "environment1"} if not assigned we will use Gantry
+                client's environment value as the default environment.
         """
         some_pred_exist = _is_not_empty(inputs) or _is_not_empty(outputs)
         pred_exist = _is_not_empty(inputs) and _is_not_empty(outputs)
@@ -621,6 +641,7 @@ class Gantry:
                 ignore_inputs=ignore_inputs,
                 timestamp=timestamp,
                 sample_rate=sample_rate,
+                tags=tags,
             )
         elif pred_exist:
             return self.log_prediction_event(
@@ -633,6 +654,7 @@ class Gantry:
                 feedback_id=feedback_id,
                 timestamp=timestamp,
                 sample_rate=sample_rate,
+                tags=tags,
             )
         elif feedback_exist:
             return self.log_feedback_event(
@@ -642,6 +664,7 @@ class Gantry:
                 feedback_version=version,
                 timestamp=timestamp,
                 sample_rate=sample_rate,
+                tags=tags,
             )
         else:
             logger.error("Tried to log record without prediction and without feedback")
@@ -658,6 +681,7 @@ class Gantry:
         ignore_inputs: Optional[List[str]] = None,
         timestamp: Optional[datetime.datetime] = None,
         sample_rate: float = 1.0,
+        tags: Optional[Dict[str, str]] = None,
     ) -> None:
         """Internal method to log an event with prediction AND feedback"""
         _check_sample_rate(sample_rate)
@@ -669,10 +693,12 @@ class Gantry:
         # don't require feedback_id if they gave us all the info to compute it
         feedback_id = {k: inputs[k] for k in (feedback_keys or [])}  # type: ignore
 
+        environment, context = self._get_env_and_context(tags)
+
         ev.update(
             _build_prediction_event(
-                self.context,
-                self.environment,
+                context,
+                environment,
                 inputs,  # type: ignore
                 outputs,
                 application,
@@ -681,11 +707,12 @@ class Gantry:
                 ignore_inputs,
                 feedback_id=None,
                 custom_timestamp=timestamp,
+                tags=tags,
             )
         )
 
         fev = _build_feedback_event(
-            self.context,
+            context,
             application,
             feedback_id,  # type: ignore
             feedback,  # type: ignore
@@ -707,6 +734,7 @@ class Gantry:
         feedback_version: Optional[Union[str, int]] = None,
         timestamp: Optional[datetime.datetime] = None,
         sample_rate: float = 1.0,
+        tags: Optional[Dict[str, str]] = None,
     ):
         """
         DEPRECATION WARNING:
@@ -746,8 +774,10 @@ class Gantry:
         if random.random() > sample_rate:
             return
 
+        _, context = self._get_env_and_context(tags)
+
         ev = _build_feedback_event(
-            self.context, application, feedback_id, feedback, feedback_version, timestamp
+            context, application, feedback_id, feedback, feedback_version, timestamp
         )
         self.log_store.log(application, ev)
 
@@ -763,6 +793,7 @@ class Gantry:
         sort_on_timestamp: bool = False,
         sample_rate: float = 1.0,
         as_batch: bool = False,
+        tags: Optional[Dict[str, str]] = None,
     ):
         """
         DEPRECATION WARNING:
@@ -801,6 +832,7 @@ class Gantry:
 
             if len(feedback_ids) == len(feedbacks):
                 events = []
+                _, context = self._get_env_and_context(tags)
 
                 for idx, timestamp in timestamp_idx:
                     try:
@@ -809,7 +841,7 @@ class Gantry:
 
                         events.append(
                             _build_feedback_event(
-                                self.context,
+                                context,
                                 application,
                                 feedback_ids[idx],
                                 feedbacks[idx],
@@ -868,6 +900,7 @@ class Gantry:
         feedback_id: Optional[dict] = None,
         timestamp: Optional[datetime.datetime] = None,
         sample_rate: float = 1.0,
+        tags: Optional[Dict[str, str]] = None,
     ):
         """
         DEPRECATION WARNING:
@@ -926,9 +959,11 @@ class Gantry:
             logger.error("inputs is not a dict.")
             return
 
+        environment, context = self._get_env_and_context(tags)
+
         ev = _build_prediction_event(
-            self.context,
-            self.environment,
+            context,
+            environment,
             inputs,
             outputs,
             application,
@@ -937,6 +972,7 @@ class Gantry:
             ignore_inputs,
             feedback_id=feedback_id,
             custom_timestamp=timestamp,
+            tags=tags,
         )
         self.log_store.log(application, ev)
 
@@ -952,10 +988,10 @@ class Gantry:
         ignore_inputs: Optional[List[str]] = None,
         feedback_ids: Optional[List[dict]] = None,
         timestamps: Optional[List[datetime.datetime]] = None,
-        reference_name: Optional[str] = None,
         sort_on_timestamp: bool = False,
         sample_rate: float = 1.0,
         as_batch: bool = False,
+        tags: Optional[Dict[str, str]] = None,
     ):
         """
         DEPRECATION WARNING:
@@ -1009,10 +1045,7 @@ class Gantry:
                         )
                     )
 
-                tags = {}
-                if reference_name:
-                    tags["env"] = "reference"
-                    tags["ref_name"] = reference_name
+                environment, context = self._get_env_and_context(tags)
 
                 timestamp_idx = _create_timestamp_idx(sort_on_timestamp, timestamps, len(inputs))
 
@@ -1021,8 +1054,8 @@ class Gantry:
                     inputs=inputs,
                     outputs=outputs,
                     timestamp_idx=timestamp_idx,
-                    context=self.context,
-                    environment=self.environment,
+                    context=context,
+                    environment=environment,
                     tags=tags,
                     version=version,
                     feedback_keys=feedback_keys,
