@@ -44,6 +44,9 @@ class BaseLogStore:
     def ping(self) -> bool:
         pass
 
+    def ready(self) -> bool:
+        pass
+
 
 class APILogStore(BaseLogStore):
     BATCH_SIZE = 20
@@ -106,6 +109,19 @@ class APILogStore(BaseLogStore):
             logger.error(f"Error during ping: {e}")
             return False
 
+    def ready(self) -> bool:
+        """
+        Checks whether the API is ready to receive traffic with
+        the provided API Key.
+        """
+        try:
+            response = self._api_client.request("GET", "/api/v1/user/me")
+            check_response(response)
+            return True
+        except Exception as e:
+            logger.error(f"Error during api key check: {e}")
+            return False
+
     def consumer_func(self, batch):
         # Catch all errors and do not raise in order
         # for thread consumer to continue running.
