@@ -10,7 +10,6 @@ from gantry.query.main import (  # noqa: F401
     list_applications,
     print_application_info,
     query,
-    set_reference,
 )
 
 __all__ = [
@@ -21,5 +20,4 @@ __all__ = [
     "list_application_versions",
     "list_application_views",
     "print_application_info",
-    "set_reference",
 ]
