@@ -1,5 +1,5 @@
 from gantry.query.core.dataframe import GantrySeries
-from gantry.query.core.utils import check_response
+from gantry.query.core.utils import check_response, same_size
 
 
 class GantryDistance(object):
@@ -10,6 +10,7 @@ class GantryDistance(object):
     def __init__(self, api_client) -> None:
         self._api_client = api_client
 
+    @same_size("feat1", "feat2")
     def d1(self, feat1: GantrySeries, feat2: GantrySeries) -> float:
         """
         Computes the D1 distance between the input feature's distributions.
@@ -21,6 +22,7 @@ class GantryDistance(object):
         """
         return self._diff_query(feat1, feat2, "d1")
 
+    @same_size("feat1", "feat2")
     def dinf(self, feat1: GantrySeries, feat2: GantrySeries) -> float:
         """
         Computes the maximum distance between the input features's distributions.
@@ -44,6 +46,7 @@ class GantryDistance(object):
         """
         return self._diff_query(feat1, feat2, "ks")
 
+    @same_size("feat1", "feat2")
     def kl(self, feat1: GantrySeries, feat2: GantrySeries) -> float:
         """
         Gets the Kullback-Leibler divergence between the input features's distributions.
