@@ -9,7 +9,7 @@ except ImportError:
 import pandas as pd
 
 from gantry.query.core.dataframe import GantrySeries
-from gantry.query.core.utils import check_response
+from gantry.query.core.utils import check_response, same_size
 
 
 def categorical(func):
@@ -141,6 +141,7 @@ class GantryMetric(object):
         return df.set_index("timestamp")
 
     @categorical
+    @same_size("outputs", "feedback")
     def accuracy_score(
         self,
         outputs: GantrySeries,
@@ -170,6 +171,7 @@ class GantryMetric(object):
         return self._prepare_pandas_dataframe(stat, results, dropna=dropna)
 
     @regression
+    @same_size("outputs", "feedback")
     def mean_squared_error(
         self,
         outputs: GantrySeries,
@@ -201,6 +203,7 @@ class GantryMetric(object):
         return self._prepare_pandas_dataframe(stat, results, dropna=dropna)
 
     @categorical
+    @same_size("outputs", "feedback")
     def confusion_matrix(
         self,
         outputs: GantrySeries,
@@ -229,6 +232,7 @@ class GantryMetric(object):
         return self._prepare_pandas_dataframe(stat, results, dropna=dropna)
 
     @categorical
+    @same_size("outputs", "feedback")
     def f1_score(
         self,
         outputs: GantrySeries,
@@ -262,6 +266,7 @@ class GantryMetric(object):
         return self._prepare_pandas_dataframe(stat, results, dropna=dropna)
 
     @regression
+    @same_size("outputs", "feedback")
     def r2_score(
         self,
         outputs: GantrySeries,
@@ -293,6 +298,7 @@ class GantryMetric(object):
         return self._prepare_pandas_dataframe(stat, results, dropna=dropna)
 
     @categorical
+    @same_size("outputs", "feedback")
     def precision_score(
         self,
         outputs: GantrySeries,
@@ -323,6 +329,7 @@ class GantryMetric(object):
         return self._prepare_pandas_dataframe(stat, results, dropna=dropna)
 
     @categorical
+    @same_size("outputs", "feedback")
     def recall_score(
         self,
         outputs: GantrySeries,
@@ -353,6 +360,7 @@ class GantryMetric(object):
         return self._prepare_pandas_dataframe(stat, results, dropna=dropna)
 
     @score
+    @same_size("outputs", "feedback")
     def roc_auc_score(
         self,
         outputs: GantrySeries,
