@@ -80,19 +80,6 @@ def query(
     return globals._Query.query(**locals())  # type: ignore[union-attr]
 
 
-@_query_alias
-def set_reference(
-    application: str,
-    version: Union[int, str],
-    start_time: datetime.datetime,
-    end_time: datetime.datetime,
-    reference_name: str,
-    tags: Optional[dict] = None,
-):
-    validate_init()
-    return globals._Query.set_reference(**locals())  # type: ignore[union-attr]
-
-
 @_query_alias
 def get_current_feedback_schema(application: str):
     validate_init()
