@@ -3,6 +3,7 @@ import logging
 import logging.handlers
 import os
 from pathlib import Path
+from queue import Empty
 from typing import Dict, List
 
 import pytest
@@ -12,6 +13,8 @@ from gantry.logger.client import Gantry
 from gantry.logger.stores import APILogStore, BaseLogStore
 from gantry.serializers import EventEncoder
 
+TIMEOUT = 0.1
+
 
 class TestingBatchIter:
     def __init__(self, queue, encode=False):
@@ -21,13 +24,35 @@ class TestingBatchIter:
     def __iter__(self):
         return self
 
-    def __next__(self):
-        if self.encode:
-            return [
-                json.dumps(self.queue.get(block=True, timeout=10), cls=EventEncoder).encode("utf8")
-            ]
+    def empty(self):
+        return self.queue.empty()
 
-        return [self.queue.get(block=True, timeout=10)]
+    def __next__(self):
+        try:
+            if self.encode:
+                return [
+                    json.dumps(
+                        self.queue.get(block=True, timeout=TIMEOUT), cls=EventEncoder
+                    ).encode("utf8")
+                ]
+
+            return [self.queue.get(block=True, timeout=TIMEOUT)]
+        except Empty:
+            return []
+
+
+class EncodedTestingBatchIter(TestingBatchIter):
+    def __init__(self, queue):
+        super().__init__(queue, True)
+
+
+class TestingBatchIterWithEmptyFlag(TestingBatchIter):
+    def __init__(self, queue, empty=True):
+        super().__init__(queue, True)
+        self._empty = empty
+
+    def empty(self):
+        return self._empty
 
 
 class TestingBatchConsumer:
@@ -47,11 +72,6 @@ class TestingBatchConsumer:
         pass
 
 
-class EncodedTestingBatchIter(TestingBatchIter):
-    def __init__(self, queue):
-        super().__init__(queue, True)
-
-
 class FileLogStore(BaseLogStore):
     """This implementation is just for testing purposes"""
 
