@@ -295,10 +295,11 @@ def test_log_feedback_event_future_timestamp(timestamp, cli_obj, caplog):
 
 
 @freeze_time(CURRENT_TIME)
+@pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("timestamp", [None, SOME_TIME])
 @pytest.mark.parametrize("feedback_version", [None, "1.2.3", 10])
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
-def test_log_feedback_event(mock_uuid, feedback_version, timestamp, cli_obj):
+def test_log_feedback_event(mock_uuid, feedback_version, timestamp, tags, cli_obj):
     mock_uuid.return_value = "12345"
     cli_obj.log_feedback_event(
         application="foobar",
@@ -306,6 +307,7 @@ def test_log_feedback_event(mock_uuid, feedback_version, timestamp, cli_obj):
         feedback={"value": "bar"},
         feedback_version=feedback_version,
         timestamp=timestamp,
+        tags=tags,
     )
     assert cli_obj.log_store.logs == [
         {
@@ -318,7 +320,7 @@ def test_log_feedback_event(mock_uuid, feedback_version, timestamp, cli_obj):
                     "feedback_id_inputs": {"A": 100},
                     "log_timestamp": CURRENT_TIME_STR,
                     "metadata": {
-                        "environment": "test",
+                        "environment": tags["env"] if tags else "test",
                         "feedback_version": feedback_version,
                         "func_name": "foobar",
                     },
@@ -506,6 +508,7 @@ def test_log_prediction_event_invalid_timestamp(timestamp, cli_obj, caplog):
 
 
 @freeze_time(CURRENT_TIME)
+@pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("timestamp", [None, SOME_TIME])
 @pytest.mark.parametrize(
     ["feedback_keys", "feedback_id"],
@@ -515,7 +518,7 @@ def test_log_prediction_event_invalid_timestamp(timestamp, cli_obj, caplog):
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
-def test_log_prediction_event(mock_uuid, feedback_keys, feedback_id, timestamp, cli_obj):
+def test_log_prediction_event(mock_uuid, feedback_keys, feedback_id, timestamp, tags, cli_obj):
     mock_uuid.return_value = "12345"
     cli_obj.log_prediction_event(
         application="foobar",
@@ -526,6 +529,7 @@ def test_log_prediction_event(mock_uuid, feedback_keys, feedback_id, timestamp,
         ignore_inputs=["A"],
         feedback_id=feedback_id,
         timestamp=timestamp,
+        tags=tags,
     )
     assert cli_obj.log_store.logs == [
         {
@@ -537,7 +541,7 @@ def test_log_prediction_event(mock_uuid, feedback_keys, feedback_id, timestamp,
                     "inputs": {},
                     "log_timestamp": CURRENT_TIME_STR,
                     "metadata": {
-                        "environment": "test",
+                        "environment": tags["env"] if tags else "test",
                         "feedback_keys": ["A"] if feedback_keys else None,
                         "func_name": "foobar",
                         "ignore_inputs": ["A"],
@@ -545,7 +549,7 @@ def test_log_prediction_event(mock_uuid, feedback_keys, feedback_id, timestamp,
                         "version": "a.b.c",
                     },
                     "outputs": {"value": "bar"},
-                    "tags": {"env": "test"},
+                    "tags": tags if tags else {"env": "test"},
                     "timestamp": CURRENT_TIME_STR if not timestamp else SOME_TIME_STR,
                 }
             ],
@@ -820,9 +824,10 @@ def test_log_record_invalid_timestamp_for_prediction(timestamp, cli_obj, caplog)
 
 
 @freeze_time(CURRENT_TIME)
+@pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("timestamp", [None, SOME_TIME])
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
-def test_log_prediction_and_feedback_event(mock_uuid, timestamp, cli_obj):
+def test_log_prediction_and_feedback_event(mock_uuid, timestamp, tags, cli_obj):
     mock_uuid.return_value = "12345"
     cli_obj._log_prediction_and_feedback_event(
         application="foobar",
@@ -831,6 +836,7 @@ def test_log_prediction_and_feedback_event(mock_uuid, timestamp, cli_obj):
         outputs={"value": "bar"},
         feedback={"value": "potato"},
         timestamp=timestamp,
+        tags=tags,
     )
 
     assert cli_obj.log_store.logs == [
@@ -842,12 +848,12 @@ def test_log_prediction_and_feedback_event(mock_uuid, timestamp, cli_obj):
                     "feedback": {"value": "potato"},
                     "inputs": {"A": 100},
                     "outputs": {"value": "bar"},
-                    "tags": {"env": "test"},
+                    "tags": tags if tags else {"env": "test"},
                     "feedback_id": "d751713988987e9331980363e24189ce",
                     "feedback_id_inputs": {},
                     "log_timestamp": CURRENT_TIME_STR,
                     "metadata": {
-                        "environment": "test",
+                        "environment": tags["env"] if tags else "test",
                         "feedback_version": "a.b.c",
                         "func_name": "foobar",
                         "provided_feedback_id": None,
@@ -1003,6 +1009,7 @@ def test_log_feedback_invalid_timestamps(timestamps, cli_obj, caplog):
 
 # TODO -> add np.ndarrays as possible feedbacks values
 @freeze_time(CURRENT_TIME)
+@pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize("timestamps", [[ANOTHER_TIME, SOME_TIME], None])
 @pytest.mark.parametrize("random_error", [True, False])
@@ -1014,7 +1021,7 @@ def test_log_feedback_invalid_timestamps(timestamps, cli_obj, caplog):
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
-def test_log_feedback(mock_uuid, feedbacks, random_error, timestamps, sort, cli_obj):
+def test_log_feedback(mock_uuid, feedbacks, random_error, timestamps, sort, tags, cli_obj):
     if random_error:
         mock_uuid.side_effect = RuntimeError
     else:
@@ -1027,6 +1034,7 @@ def test_log_feedback(mock_uuid, feedbacks, random_error, timestamps, sort, cli_
         timestamps=timestamps,
         sort_on_timestamp=sort,
         as_batch=False,
+        tags=tags,
     )
     events = [
         {
@@ -1036,7 +1044,11 @@ def test_log_feedback(mock_uuid, feedbacks, random_error, timestamps, sort, cli_
             "feedback_id": "08b84280663845e83bbf82d297013a53",
             "feedback_id_inputs": {"B": 200},
             "log_timestamp": CURRENT_TIME_STR,
-            "metadata": {"environment": "test", "feedback_version": 10, "func_name": "foobar"},
+            "metadata": {
+                "environment": tags["env"] if tags else "test",
+                "feedback_version": 10,
+                "func_name": "foobar",
+            },
             "timestamp": SOME_TIME_STR if timestamps else CURRENT_TIME_STR,
         },
         {
@@ -1046,7 +1058,11 @@ def test_log_feedback(mock_uuid, feedbacks, random_error, timestamps, sort, cli_
             "feedback_id": "5dd14615efeb2d086e519ed35efd3f73",
             "feedback_id_inputs": {"A": 100},
             "log_timestamp": CURRENT_TIME_STR,
-            "metadata": {"environment": "test", "feedback_version": 10, "func_name": "foobar"},
+            "metadata": {
+                "environment": tags["env"] if tags else "test",
+                "feedback_version": 10,
+                "func_name": "foobar",
+            },
             "timestamp": ANOTHER_TIME_STR if timestamps else CURRENT_TIME_STR,
         },
     ]
@@ -1303,27 +1319,10 @@ def test_log_predictions_invalid_timestamps(timestamps, cli_obj, caplog):
     assert "TypeError:" in caplog.text
 
 
-@pytest.mark.parametrize("reference_name", [[10], 10, 10.5, {}])
-def test_log_predictions_invalid_reference_name(reference_name, cli_obj, caplog):
-    caplog.set_level(logging.ERROR, logger="gantry.client")
-    cli_obj.log_predictions(
-        application="foobar",
-        inputs=[{"A": 100, "B": 200}],
-        outputs=[{"C": 300}],
-        feedback_keys=["A"],
-        ignore_inputs=["A", "B"],
-        version=10,
-        reference_name=reference_name,
-        as_batch=False,
-    )
-    assert "Internal exception in Gantry client" in caplog.text
-    assert "TypeError:" in caplog.text
-
-
 # TODO -> add np.ndarrays as possible inputs/outputs values
 @freeze_time(CURRENT_TIME)
+@pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("sort", [False, True])
-@pytest.mark.parametrize("reference_name", ["some-reference", None])
 @pytest.mark.parametrize("timestamps", [[ANOTHER_TIME, SOME_TIME], None])
 @pytest.mark.parametrize("random_error", [True, False])
 @pytest.mark.parametrize(
@@ -1341,9 +1340,7 @@ def test_log_predictions_invalid_reference_name(reference_name, cli_obj, caplog)
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
-def test_log_predictions(
-    mock_uuid, inputs, outputs, random_error, timestamps, reference_name, sort, cli_obj
-):
+def test_log_predictions(mock_uuid, inputs, outputs, random_error, timestamps, sort, tags, cli_obj):
     if random_error:
         mock_uuid.side_effect = RuntimeError
     else:
@@ -1356,9 +1353,9 @@ def test_log_predictions(
         ignore_inputs=["B"],
         version=10,
         timestamps=timestamps,
-        reference_name=reference_name,
         sort_on_timestamp=sort,
         as_batch=False,
+        tags=tags,
     )
 
     events = [
@@ -1369,7 +1366,7 @@ def test_log_predictions(
             "inputs": {"A": 101},
             "log_timestamp": CURRENT_TIME_STR,
             "metadata": {
-                "environment": "test",
+                "environment": tags["env"] if tags else "test",
                 "feedback_keys": ["A"],
                 "func_name": "foobar",
                 "ignore_inputs": ["B"],
@@ -1377,9 +1374,7 @@ def test_log_predictions(
                 "version": 10,
             },
             "outputs": {"C": 303},
-            "tags": {"env": "reference", "ref_name": "some-reference"}
-            if reference_name
-            else {"env": "test"},
+            "tags": tags if tags else {"env": "test"},
             "timestamp": SOME_TIME_STR if timestamps else CURRENT_TIME_STR,
         },
         {
@@ -1389,7 +1384,7 @@ def test_log_predictions(
             "inputs": {"A": 100},
             "log_timestamp": CURRENT_TIME_STR,
             "metadata": {
-                "environment": "test",
+                "environment": tags["env"] if tags else "test",
                 "feedback_keys": ["A"],
                 "func_name": "foobar",
                 "ignore_inputs": ["B"],
@@ -1397,9 +1392,7 @@ def test_log_predictions(
                 "version": 10,
             },
             "outputs": {"C": 300},
-            "tags": {"env": "reference", "ref_name": "some-reference"}
-            if reference_name
-            else {"env": "test"},
+            "tags": tags if tags else {"env": "test"},
             "timestamp": ANOTHER_TIME_STR if timestamps else CURRENT_TIME_STR,
         },
     ]
@@ -1420,8 +1413,8 @@ def test_log_predictions(
         ]
 
 
+@pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("sort", [False, True])
-@pytest.mark.parametrize("reference_name", ["some-reference", None])
 @pytest.mark.parametrize("timestamps", [[ANOTHER_TIME, SOME_TIME], None])
 @pytest.mark.parametrize(
     "outputs",
@@ -1438,9 +1431,7 @@ def test_log_predictions(
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
-def test_log_predictions_sample_rate_0_5(
-    mock_uuid, inputs, outputs, timestamps, reference_name, sort
-):
+def test_log_predictions_sample_rate_0_5(mock_uuid, inputs, outputs, timestamps, sort, tags):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
 
@@ -1454,16 +1445,16 @@ def test_log_predictions_sample_rate_0_5(
             ignore_inputs=["B"],
             version=10,
             timestamps=timestamps,
-            reference_name=reference_name,
             sort_on_timestamp=sort,
             sample_rate=0.5,
             as_batch=False,
+            tags=tags,
         )
     log_store.log_batch.assert_called_once()
 
 
+@pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("sort", [False, True])
-@pytest.mark.parametrize("reference_name", ["some-reference", None])
 @pytest.mark.parametrize("timestamps", [[ANOTHER_TIME, SOME_TIME], None])
 @pytest.mark.parametrize(
     "outputs",
@@ -1480,9 +1471,7 @@ def test_log_predictions_sample_rate_0_5(
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
-def test_log_predictions_sample_rate_0_5_none(
-    mock_uuid, inputs, outputs, timestamps, reference_name, sort
-):
+def test_log_predictions_sample_rate_0_5_none(mock_uuid, inputs, outputs, timestamps, sort, tags):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
 
@@ -1496,16 +1485,16 @@ def test_log_predictions_sample_rate_0_5_none(
             ignore_inputs=["B"],
             version=10,
             timestamps=timestamps,
-            reference_name=reference_name,
             sort_on_timestamp=sort,
             sample_rate=0.5,
             as_batch=False,
+            tags=tags,
         )
     log_store.log_batch.assert_not_called()
 
 
+@pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("sort", [False, True])
-@pytest.mark.parametrize("reference_name", ["some-reference", None])
 @pytest.mark.parametrize("timestamps", [[ANOTHER_TIME, SOME_TIME], None])
 @pytest.mark.parametrize(
     "outputs",
@@ -1522,9 +1511,7 @@ def test_log_predictions_sample_rate_0_5_none(
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
-def test_log_predictions_sample_rate_0(
-    mock_uuid, inputs, outputs, timestamps, reference_name, sort
-):
+def test_log_predictions_sample_rate_0(mock_uuid, inputs, outputs, timestamps, sort, tags):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
 
@@ -1538,10 +1525,10 @@ def test_log_predictions_sample_rate_0(
             ignore_inputs=["B"],
             version=10,
             timestamps=timestamps,
-            reference_name=reference_name,
             sort_on_timestamp=sort,
             sample_rate=0.0,
             as_batch=False,
+            tags=tags,
         )
     log_store.log_batch.assert_not_called()
 
@@ -1809,7 +1796,7 @@ def test_log_predictions_and_feedbacks_size_mismatch_versions(version, cli_obj,
 
 
 @freeze_time(CURRENT_TIME)
-@pytest.mark.parametrize("reference_name", [None, "dev"])
+@pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("timestamps", [None, [ANOTHER_TIME, SOME_TIME]])
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize("as_batch", [False, True])
@@ -1845,7 +1832,7 @@ def test_log_predictions_and_feedbacks_df(
     as_batch,
     sort,
     timestamps,
-    reference_name,
+    tags,
     cli_obj,
     log_store,
 ):
@@ -1859,11 +1846,11 @@ def test_log_predictions_and_feedbacks_df(
         feedback_keys=["A"],
         feedbacks=test_feedbacks,
         timestamps=timestamps,
-        reference_name=reference_name,
         sort_on_timestamp=sort,
         as_batch=as_batch,
+        tags=tags,
     )
-    expected_tags = {"env": "reference", "ref_name": "dev"} if reference_name else {"env": "test"}
+    expected_tags = tags if tags else {"env": "test"}
 
     event_1 = {
         "batch_id": "batch-id-123" if as_batch else None,
@@ -1874,7 +1861,7 @@ def test_log_predictions_and_feedbacks_df(
         "inputs": {"A": 100},
         "log_timestamp": CURRENT_TIME_STR,
         "metadata": {
-            "environment": "test",
+            "environment": tags["env"] if tags else "test",
             "feedback_keys": ["A"],
             "feedback_version": "2.0.1",
             "func_name": "foobar",
@@ -1896,7 +1883,7 @@ def test_log_predictions_and_feedbacks_df(
         "inputs": {"A": 101},
         "log_timestamp": CURRENT_TIME_STR,
         "metadata": {
-            "environment": "test",
+            "environment": tags["env"] if tags else "test",
             "feedback_keys": ["A"],
             "feedback_version": "2.0.1",
             "func_name": "foobar",
@@ -1967,7 +1954,6 @@ def test_log_records_preds_and_feedback(
         feedback_keys=["A"],
         feedbacks=test_feedbacks,
         timestamps=None,
-        reference_name="some-tag",
         sort_on_timestamp=False,
     )
 
@@ -2011,7 +1997,6 @@ def test_log_records_preds_and_feedback_0_5(version, test_inputs, test_outputs,
             feedback_keys=["A"],
             feedbacks=test_feedbacks,
             timestamps=None,
-            reference_name="some-tag",
             sort_on_timestamp=False,
             sample_rate=0.5,
             as_batch=False,
@@ -2057,7 +2042,6 @@ def test_log_records_preds_and_feedback_0_5_none(
             feedback_keys=["A"],
             feedbacks=test_feedbacks,
             timestamps=None,
-            reference_name="some-tag",
             sort_on_timestamp=False,
             sample_rate=0.5,
             as_batch=False,
@@ -2101,7 +2085,6 @@ def test_log_records_preds_and_feedback_0_0(version, test_inputs, test_outputs,
             feedback_keys=["A"],
             feedbacks=test_feedbacks,
             timestamps=None,
-            reference_name="some-tag",
             sort_on_timestamp=False,
             sample_rate=0.0,
             as_batch=False,
@@ -2139,7 +2122,6 @@ def test_log_records_preds_only(
         feedback_keys=["A"],
         feedbacks=None,
         timestamps=None,
-        reference_name="some-tag",
         sort_on_timestamp=False,
     )
 
@@ -2170,7 +2152,6 @@ def test_log_records_feedback_only(
         feedback_keys=["A"],
         feedbacks=test_feedbacks,
         timestamps=None,
-        reference_name="some-tag",
         sort_on_timestamp=False,
     )
 
@@ -2213,7 +2194,6 @@ def test_log_records_no_data(
         feedback_keys=["A"],
         feedbacks=test_feedbacks,
         timestamps=None,
-        reference_name="some-tag",
         sort_on_timestamp=False,
     )
 
@@ -2244,7 +2224,6 @@ def test_log_records_incomplete_preds(
         feedback_keys=["A"],
         feedbacks=test_feedbacks,
         timestamps=None,
-        reference_name="some-tag",
         sort_on_timestamp=False,
     )
 
@@ -2277,6 +2256,7 @@ def test_log_record_pred_and_feedback(mock_feedback, mock_pred, mock_pred_and_fe
         ignore_inputs=None,
         timestamp=None,
         sample_rate=1.0,
+        tags=None,
     )
     mock_pred.assert_not_called()
     mock_feedback.assert_not_called()
@@ -2313,6 +2293,7 @@ def test_log_record_pred_only(
         feedback_id=None,
         timestamp=None,
         sample_rate=1.0,
+        tags=None,
     )
     mock_feedback.assert_not_called()
 
@@ -2350,6 +2331,7 @@ def test_log_record_feedback_only(
         feedback_version="2.0.1",
         timestamp=None,
         sample_rate=1.0,
+        tags=None,
     )
 
 
@@ -2415,3 +2397,22 @@ def test_log_file(mock_line_count, test_inputs):
     )
 
     log_store.log_batch.assert_not_called()
+
+
+@pytest.mark.parametrize(
+    "tags, expected_env",
+    [
+        [None, "test"],
+        [{}, "test"],
+        [{"other_tag": "random"}, "test"],
+        [{"env": "overwrite_env"}, "overwrite_env"],
+    ],
+)
+def test_get_env_and_context(tags, expected_env):
+    log_store = mock.MagicMock()
+    gantry = client.Gantry(log_store=log_store, environment="test")
+
+    environment, context = gantry._get_env_and_context(tags)
+
+    assert environment == expected_env
+    assert context == {"environment": expected_env}
