@@ -1,6 +1,8 @@
 from queue import Queue
 
+import mock
 import pytest
+from freezegun import freeze_time
 
 from gantry.logger import consumer
 
@@ -124,6 +126,51 @@ def test_batch_iter_builder_timeout(q):
     assert q.empty()
 
 
+def test_batch_iter_builder_flushing_after(q):
+    """Check that batch iterator flushes after cache timeout"""
+    batch_iter = consumer._BatchIterBuilder(
+        queue=q,
+        batch_size_record_limit=100,
+        single_event_limit_bytes=100,
+        batch_size_limit_bytes=100,
+        timeout_secs=0.01,
+        flush_after_secs=-1,
+    )
+
+    it = iter(batch_iter)
+
+    q.put("1")
+    q.put("2")
+
+    assert next(it) == []
+    assert next(it) == [b'"1"']
+    assert next(it) == [b'"2"']
+
+
+@freeze_time("2012-01-14 03:21:34")
+def test_batch_iter_flush_and_reset(q):
+    """Check that this method flushed, marks
+    the task as done and reset the _last_flushed ckpt.
+    """
+    batch_iter = consumer._BatchIterBuilder(
+        queue=q,
+        batch_size_record_limit=100,
+        single_event_limit_bytes=100,
+        batch_size_limit_bytes=10,
+        timeout_secs=0.1,
+    )
+    q.task_done = mock.Mock()
+
+    batch_iter.current_json_dumped_batch = ["foobar"]
+    ret = batch_iter._flush_and_reset("barbaz")
+
+    q.task_done.assert_called_once()
+    assert ret == ["foobar"]
+    assert batch_iter.current_json_dumped_batch == ["barbaz"]
+    assert batch_iter.current_json_dumped_batch_size_bytes == 6
+    assert batch_iter._last_flushed == 1326511294
+
+
 def test_batch_iter_empty(q):
     """Batch iterator is empty if the queue
     is empty and there are no events in the cache
