@@ -10,6 +10,7 @@ from gantry.logger import main
     ["method", "params"],
     [
         ("ping", {}),
+        ("ready", {}),
         ("instrument", {}),
         ("log_feedback", {}),
         ("log_feedback_event", {}),
@@ -30,6 +31,7 @@ def test_uninit_client_main(module, method, params):
     ["method", "params"],
     [
         ("ping", {}),
+        ("ready", {}),
         ("instrument", {}),
         ("log_feedback", {}),
         ("log_feedback_event", {}),
@@ -47,10 +49,10 @@ def test_uninit_client_main(module, method, params):
                 "feedbacks": [4, 5, 6],
                 "ignore_inputs": ["A"],
                 "timestamps": "today",
-                "reference_name": "barbaz",
                 "sort_on_timestamp": True,
                 "sample_rate": 1.0,
                 "as_batch": False,
+                "tags": {},
             },
         ),
         (
@@ -66,6 +68,7 @@ def test_uninit_client_main(module, method, params):
                 "ignore_inputs": ["A"],
                 "timestamp": "today",
                 "sample_rate": 1.0,
+                "tags": {},
             },
         ),
     ],
@@ -76,3 +79,17 @@ def test_logger_methods(module, method, params):
     with mock.patch("gantry.logger.main._CLIENT", m):
         getattr(module, method)(**params)
         getattr(m, method).assert_called_once_with(**params)
+
+
+@pytest.mark.parametrize("ping_ret", [True, False])
+@pytest.mark.parametrize("module", [main, gantry])
+@mock.patch("gantry.logger.client.Gantry.ping")
+@mock.patch("gantry.logger.client.Gantry.ready")
+def test_logger_init(mock_ready, mock_ping, module, ping_ret):
+    m = mock.Mock()
+    mock_ping.return_value = ping_ret
+    with mock.patch("gantry.logger.main._CLIENT", m):
+        module.init()
+        mock_ping.assert_called_once()
+        if ping_ret:
+            mock_ready.assert_called_once()
