@@ -60,6 +60,27 @@ def test_ping_success(status_code):
         assert store.ping() is True
 
 
+@pytest.mark.parametrize("status_code", [200, 400, 401, 404, 500])
+def test_ready_fail(status_code):
+    store = APILogStore(TEST_HOST)
+    with responses.RequestsMock() as rsps:
+        rsps.add(responses.GET, f"{TEST_HOST}/api/v1/user/me", status=status_code)
+        assert store.ready() is False
+
+
+@pytest.mark.parametrize("status_code", [200])
+def test_ready_success(status_code):
+    store = APILogStore(TEST_HOST)
+    with responses.RequestsMock() as rsps:
+        rsps.add(
+            responses.GET,
+            f"{TEST_HOST}/api/v1/user/me",
+            status=status_code,
+            json={"response": "ok"},
+        )
+        assert store.ready() is True
+
+
 @pytest.mark.parametrize(
     ["data", "as_raw"], [(["foo", "bar", "baz"], False), ([b'"foo"', b'"bar"', b'"baz"'], True)]
 )
