@@ -15,6 +15,7 @@ from gantry.query.core.utils import (
     get_last_application_version,
     get_start_end_time_from_view,
     runs_on,
+    same_size,
 )
 
 from ..conftest import DATA, END_TIME, END_TIME_STR, ORIGIN, START_TIME, START_TIME_STR
@@ -88,6 +89,48 @@ def test_runs_on_valid(supported, valid):
     assert f(mock.Mock(dtype=valid))
 
 
+l_1 = list(range(1))
+l_10 = list(range(10))
+l_100 = list(range(100))
+
+
+@pytest.mark.parametrize(
+    ["args", "kwargs"],
+    [
+        ([l_10, l_10], {"s3": l_10}),
+        ([l_10], {"s3": l_10, "s2": l_10}),
+        ([], {"s1": l_10, "s3": l_10, "s2": l_10}),
+    ],
+)
+def test_same_size_valid(args, kwargs):
+    dec = same_size("s1", "s2", "s3")
+    f = dec(lambda s1, s2, s3: True)
+    assert f(*args, **kwargs)
+
+
+@pytest.mark.parametrize(
+    ["args", "kwargs"],
+    [
+        ([l_10, l_100], {"s3": l_10}),
+        ([l_10], {"s3": l_100, "s2": l_100}),
+        ([], {"s1": l_10, "s3": l_100, "s2": l_10}),
+        ([], {"s1": l_10, "s3": l_100, "s2": l_1}),
+    ],
+)
+def test_same_size_error(args, kwargs):
+    dec = same_size("s1", "s2", "s3")
+    f = dec(lambda s1, s2, s3: True)
+    with pytest.raises(ValueError):
+        f(*args, **kwargs)
+
+
+@pytest.mark.parametrize("bad_params", [["other"], ["s1", "s2", "s3"]])
+def test_same_size_bad_params(bad_params):
+    dec = same_size(*bad_params)
+    with pytest.raises(ValueError):
+        _ = dec(lambda s1, s2: True)
+
+
 @pytest.mark.parametrize(
     ["start_time", "end_time", "version", "content"],
     [
